"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.safeRequire = exports.getUserSource = exports.isObj = exports.mergeDeep = exports.TCBError = exports.getDsFullName = exports.dash2smallCamelCase = exports.lowercaseKey = exports.capitalizeKey = exports.getCloudbaseContext = exports.getConfig = exports.setConfig = void 0;
const node_sdk_1 = __importDefault(require("@cloudbase/node-sdk"));
/**
 * node-sdk 内部共享的配置信息
 *  !!TODO: 在云函数中, 单进程, 信息不共享, 不会出现数据串的问题, 但放到云托管中, 则会有问题
 */
const config = {};
/**
 * 设置配置, 应当只在 get-fn-context 中调用
 */
function setConfig(cfg) {
    Object.assign(config, cfg);
}
exports.setConfig = setConfig;
function getConfig(key) {
    return key ? config[key] : config;
}
exports.getConfig = getConfig;
// 缓存解析的完整的云函数context信息
let parsedContext;
/**
 * 解析云函数的 context
 *  为保证能获取到完整的context信息, 应当保证在第一次调用时有传入 context
 * @param context tcb 云函数的context, 即 main 函数的第二个参数
 * @param isTestMode 是否为测试模式, 在环境信息中注入 isTestMode
 */
function getCloudbaseContext(context, isTestMode) {
    if (context) {
        parsedContext = Object.assign({ isTestMode }, node_sdk_1.default.getCloudbaseContext(context));
        return parsedContext;
    }
    return parsedContext || node_sdk_1.default.getCloudbaseContext();
}
exports.getCloudbaseContext = getCloudbaseContext;
/**
 * 将对象的key的首字母转换为大写, 适应 yunapi 的命名风格
 *  { userName: 'xxx', id: 'xxx' } => { UserName: 'xxx', Id: 'xxxx'}
 */
function capitalizeKey(obj) {
    const result = {};
    return Object.keys(obj).reduce((acc, key) => {
        const capitalized = key.charAt(0).toUpperCase() + key.slice(1);
        acc[capitalized] = obj[key];
        return acc;
    }, result);
}
exports.capitalizeKey = capitalizeKey;
/**
 * 将对象的 key 转换为全小写
 *  for http header usage
 */
function lowercaseKey(obj) {
    const result = {};
    return Object.keys(obj).reduce((acc, cur) => {
        acc[cur.toLowerCase()] = obj[cur];
        return acc;
    }, result);
}
exports.lowercaseKey = lowercaseKey;
/**
 * 将破折号转换为小驼峰命名方式
 *  dash-name => dashName
 */
function dash2smallCamelCase(str) {
    return str.replace(/^-|-$/g, '').replace(/-([^-])/g, ($0, $1) => $1.toUpperCase());
}
exports.dash2smallCamelCase = dash2smallCamelCase;
/**
 * 获取数据源完整名称, 该名称用于发布后的tcb云函数, 也用于tcb的数据库名称
 * @param ds 数据源关键信息
 * @param isPreview 是否预览
 * @returns 数据源表名
 */
function getDsFullName(ds, isPreview) {
    return `lcap-${ds.id}-${ds.name}${isPreview ? '-preview' : ''}`;
}
exports.getDsFullName = getDsFullName;
/** 预置的错误对象, 可用该对象抛出自定义错误代码及错误信息 */
class TCBError extends Error {
    constructor(code, message) {
        super(message);
        this.code = code;
        this.name = 'TCBError';
    }
}
exports.TCBError = TCBError;
/* eslint-disable no-param-reassign */
function isObject(obj) {
    return obj && typeof obj === 'object';
}
/**
 * Performs a deep merge of `source` into `target`.
 * Mutates `target` only but not its objects and arrays.
 *
 * @author inspired by [jhildenbiddle](https://stackoverflow.com/a/48218209).
 */
function mergeDeep(target, source) {
    if (!isObject(target) || !isObject(source)) {
        return source;
    }
    Object.keys(source).forEach((key) => {
        const targetValue = target[key];
        const sourceValue = source[key];
        if (Array.isArray(targetValue) && Array.isArray(sourceValue)) {
            target[key] = targetValue.concat(sourceValue);
        }
        else if (isObject(targetValue) && isObject(sourceValue)) {
            target[key] = mergeDeep(Object.assign({}, targetValue), sourceValue);
        }
        else {
            target[key] = sourceValue;
        }
    });
    return target;
}
exports.mergeDeep = mergeDeep;
/** 是否为对象(非数组) */
function isObj(obj, key) {
    const target = obj && key ? obj[key] : obj;
    if (!target)
        return false;
    return !Array.isArray(target) && typeof target === 'object';
}
exports.isObj = isObj;
/** 获取用户来源信息 */
function getUserSource() {
    const envInfo = getCloudbaseContext();
    // @ts-ignore
    const openId = envInfo.WX_FROM_OPENID || envInfo.WX_OPENID;
    const uid = envInfo.TCB_UUID;
    if (!openId && !uid) {
        // 测试模式固定用 wedaId 0
        // @ts-ignore
        if (envInfo.isTestMode) {
            return {
                uid: '0',
                source: 4,
            };
        }
        return;
    }
    return {
        uid: openId || uid,
        /** uid 类型, 1.tcb 2.微信 3.企业微信 4.weda内部 */
        source: openId ? 2 : 1,
    };
}
exports.getUserSource = getUserSource;
/**
 * 安全require, 如果引入的路径不存在, 或者文件内容语法错误, 则返回undefined, 而不会crash整个代码
 */
function safeRequire(filePath, needErrorStack) {
    try {
        // eslint-disable-next-line @typescript-eslint/no-require-imports
        return require(filePath);
    }
    catch (error) {
        console.log('failed to require', filePath, error);
        if (needErrorStack)
            return error.stack || error.message;
    }
}
exports.safeRequire = safeRequire;
